using System;
using System.Data;
using System.Web;
using System.Collections.Generic;
using System.Web.Security;
using System.Security.Principal;

using Common;
using DataLayer;

namespace BusinessLayer
{
	public class Administration
    {
        #region Fields
        public static int MinPasswordLength
        {
            get 
            {
                int len = ParseMethods.ReadIntZero(ApplicationConfiguration.AppSettings[StringConstants.CONFIG_KEY_MIN_PASSWORD_LENGTH]);
                if (len <= 0)
                    len = 5;

                return len;
            }
        }
		private static int timeout = 30;
        #endregion

        #region Methods
        public static void AuthenticateUser(UserData user)
		{
			if (user.LoginStatus == LoginStatus.Authenticated)
			{
				FormsAuthenticationTicket authTicket = new FormsAuthenticationTicket(1, user.UserID.ToString(), DateTime.Now, DateTime.Now.AddMinutes(timeout), false, System.Web.HttpContext.Current.Session.SessionID);
				string encryptedTicket = FormsAuthentication.Encrypt(authTicket);

				HttpCookie authCookie = new HttpCookie(System.Web.HttpContext.Current.Session.SessionID, encryptedTicket);
				authCookie.Expires = authTicket.Expiration;
				System.Web.HttpContext.Current.Response.Cookies.Add(authCookie);
				///
				System.Web.HttpContext.Current.Cache.Insert(encryptedTicket, user.UserID, null, DateTime.Now.AddMinutes(timeout), TimeSpan.Zero);
				///

                System.Web.HttpContext.Current.Session.Clear();
                UserData.SetCurrentUser(user);
			}
		}

		public static bool RenewAuthenticationTicket(string userID, HttpCookie session)
		{
			if (userID == null || userID.Trim() == "")
				return false;

			FormsAuthenticationTicket authTicket = new FormsAuthenticationTicket(1, userID, DateTime.Now, DateTime.Now.AddMinutes(timeout), false, session.Value);
			string encryptedTicket = FormsAuthentication.Encrypt(authTicket);
			HttpCookie authCookie = new HttpCookie(session.Value, encryptedTicket);
			authCookie.Expires = authTicket.Expiration;
			System.Web.HttpContext.Current.Response.Cookies.Add(authCookie);			
			return true;
		}

        public static UserData Login(int userID, string password)
        {
            UserData user;
            try
            {
                DataTable dtWebUserInfo = BusinessHelper.Person.CheckPassword(userID, password);

                if (dtWebUserInfo.Rows.Count > 0)
                {
                    user = new UserData(userID);
                    
                    user.WebUserInfo.Add(password, dtWebUserInfo);
                    user.Reload();
                    user.SetLoginStatus(LoginStatus.Authenticated);
                    AuthenticateUser(user);
                }
                else
                    user = new UserData();

            }
            catch (Exception exc)
            {
                Common.ApplicationLog.LogException(exc, "Handled exception: Login failed");
                user = new UserData();
            }

            return user;
        }

		public static bool Logout()
		{
			try
			{
				FormsAuthentication.SignOut();
				System.Web.HttpContext.Current.Session.Abandon();

                HttpCookie cookie = new HttpCookie(System.Web.HttpContext.Current.Session.SessionID);
                System.Web.HttpContext.Current.Response.Cookies.Add(cookie);

				return true;
			}
			catch (Exception exc)
			{
				Common.ApplicationLog.LogException(exc, "Handled exception: Logout failed");
				return false;
			}
        }
        #endregion
    }
}
