using System;
using System.Data;
using System.Collections;

using Common;
using DataLayer;

namespace BusinessLayer
{
	public class BusinessHelper
	{
		public static void SetDataLayer(ConnectionParams connectionParams)
		{
			MainDALC.Initialize(connectionParams);
		}

        public class Hospital
        {
            public static DataTable GetHospitalDetails()
            {
                DataTable hospitalDetails = MSSqlQueries.Hospital.GetHospitalDetails();
                return hospitalDetails;
            }
        }

        public class Person
        {
            public static DataTable GetPersonalDetails(int personID, string password)
            {
                DataTable data = MSSqlQueries.Person.GetPersonalDetails(personID, password);
                return data;
            }

            internal static DataTable GetRequestDates(int personID)
            {
                DataTable data = MSSqlQueries.Person.GetRequestDates(personID);
                return data;
            }

            public static DataTable GetResultRequestDates(DataTable dtRequestResults)
            {
                return DataTableHelper.SelectDistinct(dtRequestResults, new string[] { DBS.F_REQUEST_REQUEST_DATE_TIME });
            }

            public static DataTable GetRequestDatesWithAuthorization(UserData user)
            {
                DataTable data = GetRequestDates(user.UserID);
                data.Columns.Add(DBS.F_AUTHORIZED, typeof(bool));

                if (data.Rows.Count > 0)
                {
                    DateTime requestDate;
                    bool isAuth;
                    foreach (DataRow drRequest in data.Rows)
                    {
                        if (drRequest.IsNull(DBS.F_REQUEST_REQUEST_DATE_TIME))
                            isAuth = false;
                        else
                        {
                            requestDate = ParseMethods.ReadDate(drRequest[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                            isAuth = user.AuthorizeDate(requestDate);
                        }

                        drRequest[DBS.F_AUTHORIZED] = isAuth;
                    }
                }

                return data;
            }

            public static DataTable GetResults(ResultsFilter resultsFilter, UserData user)
            {
                if (resultsFilter == null || user.UserID <= 0)
                    return null;

                DataTable data = new DataTable();
                
                WebUserInfo wi;
                ResultsFilter filter;
                DateTime requestDate;
                //Makes search in DB foreach validated webuser Info 

                if (Validation.IsValidDate(resultsFilter.RequstDate))
                {
                    filter = new ResultsFilter(resultsFilter);
                    wi = user.WebUserInfo[resultsFilter.RequstDate];
                    if (wi != null)
                    {
                        filter.Password = wi.Password;
                        MSSqlQueries.Person.FillResults(data, filter);
                    }
                }
                else
                {
                    DataTable dtRequestDates = GetRequestDatesWithAuthorization(user);
                    foreach (DataRow drAuthRequest in dtRequestDates.Rows)
                    {
                        if (!ParseMethods.ReadBool(drAuthRequest[DBS.F_AUTHORIZED]))
                            continue;

                        requestDate = ParseMethods.ReadDate(drAuthRequest[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                        wi = user.WebUserInfo[requestDate];
                        if (wi==null)
                            continue;

                        filter = new ResultsFilter(resultsFilter);
                        filter.Password = wi.Password;
                        filter.RequstDate = requestDate;

                        MSSqlQueries.Person.FillResults(data, filter);
                    }

                }

                return data;
            }

            public static DataTable GetServices(int personID)
            {
                DataTable data = MSSqlQueries.Person.GetServices(personID);
                return data;
            }

            public static int CreateUser(int personID, string password, DateTime startDate, DateTime endDate)
            {
                int result = MSSqlQueries.Person.CreateUser(personID, password, startDate, endDate);
                return result;
            }

            public static int ChangePassword(int webUserID, int personID, string oldPassword, string newPassword)
            {
                int result = MSSqlQueries.Person.ChangePassword(webUserID, personID, oldPassword, newPassword);
                return result;
            }

            public static DataTable CheckPassword(int personID, string password)
            {
                DataTable data = MSSqlQueries.Person.CheckPassword(personID, password);
                return data;
            }                       
        }
	}
}