using System;
using System.Collections;
using System.Data;
using System.Collections.Generic;
using System.Text;

using Common;
using DataLayer;

namespace BusinessLayer
{
	public class UserData
	{
		#region Fields		
 		private int userID;
		private string firstName;
		private string secondName;
        private string lastName;
		private string phone;
        private string egn;
		private string address;

        private UserRole role;
		private PageHistory pageHistory;
        private LoginStatus loginStatus;
        private WebUserInfoCollection webUserInfo;
        private PageCache pageCache; 
		#endregion

		#region Properties
        public WebUserInfoCollection WebUserInfo
        {
            get { return webUserInfo; }        
        }
		public PageHistory PageHistory
		{
			get 
			{
				return pageHistory;
			}
			set 
			{
				pageHistory = value;
			}
		}

        public PageCache PageCache
        {
            get
            {
                return pageCache;
            }
        }

		public static UserData CurrentUser
		{
			get
			{
				UserData user = System.Web.HttpContext.Current.Session[StringConstants.SESSION_KEY_USER_DATA] as UserData;
				if (user == null)
				{
					user = new UserData();
					System.Web.HttpContext.Current.Session.Add(StringConstants.SESSION_KEY_USER_DATA, user);
				}

				return user;
			}
		}

		public LoginStatus LoginStatus
		{
			get 
			{
				return loginStatus;
			}
		}

		public int UserID
		{
			get
			{
				return userID;
			}
		}

		public string FirstName
        {
            get 
            {
                return firstName;
            }
        }

        public string SecondName
        {
            get
            {
                return secondName;
            }
        }

        public string LastName
        {
            get
            {
                return lastName;
            }
        }

		public UserRole Role
		{
			get 
			{
				return role;
			}
		}

        public string Phone
        {
            get
            {
                return phone;
            }
        }

        public string EGN
        {
            get
            {
                return egn;
            }
        }

        public string Address
        {
            get
            {
                return address;
            }
        }

        public string FullName
        {
            get 
            {
                return firstName + " " + lastName;
            }
        }

		public bool IsAdministrator
		{
			get 
			{
				return role == UserRole.Admin;
			}
		}

		public bool IsUser
		{
			get
			{
				return role == UserRole.User;
			}
		}
                      
		#endregion

		#region Constructors/Initializers
        public UserData()
        {
            role = UserRole.None;
            pageHistory = new PageHistory();
            loginStatus = LoginStatus.NotAuthenticated;
            webUserInfo = new WebUserInfoCollection();
            pageCache = new PageCache();
        }

        public UserData(int userID) : this()
        {
            this.userID = userID;
            Reload();
        }

		private void Init(DataRow userRow)
		{
            this.firstName = userRow[DBS.F_PERSON_FIRST_NAME].ToString();
            this.secondName = userRow[DBS.F_PERSON_SECOND_NAME].ToString();
            this.lastName = userRow[DBS.F_PERSON_LAST_NAME].ToString();
            this.egn = userRow[DBS.F_PERSON_EGN].ToString();
            this.phone = userRow[DBS.F_PERSON_PHONE].ToString();
            this.address = userRow[DBS.F_PERSON_ADDRESS].ToString();
            this.role = UserRole.User;
		}
		#endregion

		#region Methods
        public bool ChangePassword(int webUserID, string oldPassword, string newPassword)
        {
            int result = BusinessHelper.Person.ChangePassword(webUserID, this.userID, oldPassword, newPassword);
            if (result>0)
            {
                if (webUserID > 0)
                    WebUserInfo.Remove(webUserID);
                else
                    WebUserInfo.Remove(oldPassword);

                DataTable dtWebUserInfo = BusinessHelper.Person.CheckPassword(userID, newPassword);
                this.WebUserInfo.Add(newPassword, dtWebUserInfo);
                
                return true;
            }
            return false;
        }
		public void Reload()
		{
			if (userID <= 0 || webUserInfo.Count == 0) 
                return;

            WebUserInfo wi;
            DataTable dtPersonalData;
            foreach (int key in webUserInfo.Items.Keys)
            {
                wi = webUserInfo.Items[key] as WebUserInfo;
                if (wi != null)
                {
                    dtPersonalData = BusinessHelper.Person.GetPersonalDetails(userID, wi.Password);
                    if (dtPersonalData.Rows.Count == 1)
                        Init(dtPersonalData.Rows[0]);
                    break;
                }
            }
		}

		internal void SetLoginStatus(LoginStatus status)
		{
			this.loginStatus = status;
		}

        internal static void SetCurrentUser(UserData user)
        {
            System.Web.HttpContext.Current.Session.Add(StringConstants.SESSION_KEY_USER_DATA, user);
        }

        public bool IsInRole(UserRole role)
        {
            return IsInRole((int)role);
        }

        public bool IsInRole(int role)
        {
            return (int)this.role == role;
        }

        public bool IsInRoles(IList roles)
        {
            bool res = false;

            foreach (string role in roles)
            {
                if (IsInRole(ParseMethods.ReadInt(role)))
                {
                    res = true;
                    break;
                }
            }

            return res;
        }

        public DataTable GetWebUserInfoTable()
        {
            WebUserInfoCollection wInfo = this.WebUserInfo;
            DataTable data = MSSqlQueries.Person.CheckPassword(0, "");
            data.Columns.Add(DBS.F_NAME, typeof(string));
            
            DataRow row;
            WebUserInfo wi;
            foreach (int key in wInfo.Items.Keys)
            {
                wi = wInfo.Items[key] as WebUserInfo;
                row = data.NewRow();

                row[DBS.F_WEBUSERS_PERSONID] = userID;
                row[DBS.F_WEBUSERS_STARTDATE] = wi.StartDate;
                row[DBS.F_WEBUSERS_ENDDATE] = wi.EndDate;
                row[DBS.F_WEBUSERS_PWD] = wi.Password;
                row[DBS.F_WEBUSERS_WEBUSERID] = wi.ID;
                row[DBS.F_NAME] = FormatMethods.GetStringDateBg(wi.StartDate) + "-" + FormatMethods.GetStringDateBg(wi.EndDate);

                data.Rows.Add(row);
            }
            data.DefaultView.Sort = DBS.F_WEBUSERS_STARTDATE;
            return data;
        }

        public void UnAuthorizeDate(DateTime date)
        {
            WebUserInfo.Remove(date);
        }

        public bool AuthorizeDate(DateTime date)
        {
            WebUserInfo wi;
            foreach (int key in WebUserInfo.Items.Keys)
            {
                wi = WebUserInfo[key];
                if (wi.IsAuthorized(date))
                    return true;
            }
            return false;
        }

        public bool AuthorizeDate(DateTime date, string password)
        {
            bool dateIsValid = Validation.IsValidDate(date);
            if (dateIsValid && AuthorizeDate(date))
                return true;

            DataTable dtWebUserInfo = BusinessHelper.Person.CheckPassword(userID, password);
            bool isAuth = false;
            if (dtWebUserInfo.Rows.Count > 0)
            {
                foreach (DataRow row in dtWebUserInfo.Rows)
                {
                    if (! dateIsValid || 
                        (dateIsValid
                        && ParseMethods.ReadDate(row[DBS.F_WEBUSERS_STARTDATE]) <= date
                        && ParseMethods.ReadDate(row[DBS.F_WEBUSERS_ENDDATE]) > date))
                    {
                        isAuth = true;
                        WebUserInfo.Add(password, row);                        
                    }
                }
            }

            return isAuth;
        }

        public string DateFormat
        {
            get { return "dd/MM/yyyy";  }
        }

        public string CalendarDateFormat
        {
            get 
            {
                return "%d/%m/%Y";
            }
        }

		#endregion
	}
}
