using System;
using System.IO;
using System.Text;
using System.Security.Cryptography;

namespace Common
{
	public class Encryption
	{
		#region Constants
		private const string defaultKey = "cccccccccccccccc";
		private const string defaultIV = "dddddddddddddddd";
		#endregion

		#region Fields
		/// <summary>
		/// Used to set the secret key for the symmetric algorithm
		/// </summary>
		private string strKey = "cccccccccccccccc";
		/// <summary>
		/// Used to set the initialization vector (IV) for the symmetric algorithm.
		/// </summary>
		private string strIV = "dddddddddddddddd"; //must be 16 bytes exactly
		#endregion

		#region Help Methods
		/// <summary>
		/// Returns the hex representation of an integer between 0 and 15
		/// </summary>
		/// <param name="val"></param>
		/// <returns></returns>
		public static char Hex(int val)
		{
			switch (val)
			{
				case 0:
					return '0';
				case 1:
					return '1';
				case 2:
					return '2';
				case 3:
					return '3';
				case 4:
					return '4';
				case 5:
					return '5';
				case 6:
					return '6';
				case 7:
					return '7';
				case 8:
					return '8';
				case 9:
					return '9';
				case 10:
					return 'A';
				case 11:
					return 'B';
				case 12:
					return 'C';
				case 13:
					return 'D';
				case 14:
					return 'E';
				case 15:
					return 'F';
				default:
					return '0';
			}
		}

		/// <summary>
		/// Returns a HEX digit from an integer between 0 and 15
		/// </summary>
		/// <param name="val"></param>
		/// <returns></returns>
		public static int FromHex(char val)
		{
			switch (val)
			{
				case 'A':
				case 'a':
					return 10;
				case 'b':
				case 'B':
					return 11;
				case 'c':
				case 'C':
					return 12;
				case 'd':
				case 'D':
					return 13;
				case 'e':
				case 'E':
					return 14;
				case 'f':
				case 'F':
					return 15;
				case '1':
					return 1;
				case '2':
					return 2;
				case '3':
					return 3;
				case '4':
					return 4;
				case '5':
					return 5;
				case '6':
					return 6;
				case '7':
					return 7;
				case '8':
					return 8;
				case '9':
					return 9;
				default:
					return 0;
			}
		}
		#endregion

		public Encryption(string initVector)
		{
			//must be 16 bytes exactly
			if (initVector.Length < 16)
				return;			//use default;

			char[] newIV = new char[16];
			for (int i = 0; i < 16; i++)
				newIV[i] = (char)((byte)defaultIV[i] ^ (byte)initVector[i]);

			strIV = new string(newIV);
		}

		/// <summary>
		/// Gets a hex string representation of a byte array
		/// </summary>
		/// <param name="arrBytes"></param>
		/// <returns></returns>
		public static string GetStringFromBytes(byte[] arrBytes)
		{
			char[] chars = new char[arrBytes.Length * 2];

			for (int i = 0; i < arrBytes.Length; i++)
			{
				chars[2 * i] = Hex((char)(arrBytes[i] / 16));
				chars[2 * i + 1] = Hex((char)(arrBytes[i] % 16));
			}

			string result = new string(chars);
			return result;
		}

		private static string PasswordInitVector()
		{
			return (Math.PI * Math.E * 1E16).ToString("G17");
		}

		public static string EncryptPassword(string passwd)
		{
			Encryption encryptor = new Encryption(PasswordInitVector());
			passwd = encryptor.Encrypt(passwd);
			if (passwd.Length > 40)
				passwd = passwd.Substring(0, 40);
			return passwd;
		}

		public static string DecryptPassword(string passwd)
		{
			Encryption encryptor = new Encryption(PasswordInitVector());
			passwd = encryptor.Decrypt(passwd);
			return passwd;
		}

		/// <summary>
		/// Parses a hex string to obtain a byte array
		/// </summary>
		/// <param name="strHex"></param>
		/// <returns></returns>
		public static byte[] GetBytesFromString(string strHex)
		{
			byte[] arrBytes = new byte[strHex.Length / 2];
			char[] chars = strHex.ToCharArray();
			for (int i = 0; i < strHex.Length / 2; i++)
			{
				arrBytes[i] = (byte)(FromHex(chars[i * 2]) * 16 + FromHex(chars[i * 2 + 1]));
			}

			return arrBytes;
		}


		public string Encrypt(string toEncrypt)
		{
			ASCIIEncoding asciiConverter = new ASCIIEncoding();
			UTF8Encoding utf8Converter = new UTF8Encoding();
			RijndaelManaged mRijndael = new RijndaelManaged();
			mRijndael.Padding = PaddingMode.Zeros;

			mRijndael.Key = asciiConverter.GetBytes(strKey);
			mRijndael.IV = asciiConverter.GetBytes(strIV);

			ICryptoTransform encryptor = mRijndael.CreateEncryptor(mRijndael.Key, mRijndael.IV);
			MemoryStream msEncrypt = new MemoryStream();
			CryptoStream csEncrypt = new CryptoStream(msEncrypt, encryptor, CryptoStreamMode.Write);

			byte[] toEncryptBytes = utf8Converter.GetBytes(toEncrypt);
			csEncrypt.Write(toEncryptBytes, 0, toEncryptBytes.Length);
			csEncrypt.FlushFinalBlock();

			byte[] encryptedBytes = msEncrypt.ToArray();

			string result = GetStringFromBytes(encryptedBytes);
			return result;
		}

		public string Decrypt(string toDecrypt)
		{
			byte[] encBytes = GetBytesFromString(toDecrypt);
			ASCIIEncoding asciiConverter = new ASCIIEncoding();
			UTF8Encoding utf8Converter = new UTF8Encoding();
			RijndaelManaged mRijndael = new RijndaelManaged();
			mRijndael.Padding = PaddingMode.Zeros;

			mRijndael.Key = asciiConverter.GetBytes(strKey);
			mRijndael.IV = asciiConverter.GetBytes(strIV);

			ICryptoTransform decryptor = mRijndael.CreateDecryptor(mRijndael.Key, mRijndael.IV);
			MemoryStream msDecrypt = new MemoryStream(encBytes);
			CryptoStream csDecrypt = new CryptoStream(msDecrypt, decryptor, CryptoStreamMode.Read);

			byte[] decryptedBytes = new byte[asciiConverter.GetBytes(toDecrypt).Length];
			csDecrypt.Read(decryptedBytes, 0, decryptedBytes.Length);

			int i = decryptedBytes.Length - 1;
			while ((i >= 0) && (decryptedBytes[i] == 0)) i--;
			i++;
			byte[] newArray = new byte[i];
			Array.Copy(decryptedBytes, 0, newArray, 0, i);

			string result = utf8Converter.GetString(newArray);

			return result;
		}

		/// <summary>
		/// This class provides one way hashing using the MD5 algorithm
		/// </summary>
		public class HashProvider
		{
			MD5CryptoServiceProvider md5;
			/// <summary>
			/// Default constructor. Initializes the MD5 crypto service provider
			/// </summary>
			public HashProvider()
			{
				md5 = new MD5CryptoServiceProvider();
			}

			/// <summary>
			/// Computes hash of an ascii string and returns the hash in hexadecimal or ASCII string.
			/// </summary>
			/// <param name="asciiString"></param>
			/// <param name="hex"></param>
			/// <returns></returns>
			public string ComputeHash(string asciiString, bool hex)
			{
				byte[] strBytes = System.Text.Encoding.ASCII.GetBytes(asciiString);
				byte[] hash = md5.ComputeHash(strBytes);
				string strHashed = hex ? Encryption.GetStringFromBytes(hash) : System.Text.Encoding.ASCII.GetString(hash);
				return strHashed;
			}
		}
	}	
}
