using System;

namespace Common
{
	public class FormatMethods
	{
		public static string GetClientStringDate(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";

			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";

			return val.ToShortDateString();
		}
		
		public static string GetStringYear(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";
			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";
			return val.Year.ToString();
		}

		public static string GetStringMonth(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";
			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";
			return val.Year.ToString() + '/' + val.Month.ToString();
		}

		public static string GetStringWeek(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";
			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";
			int week = GetWeekNumber(val);
			return val.Year.ToString() + '/' + week.ToString();
		}

		public static int GetWeekNumber(DateTime date)
		{
			int weekNr = System.Globalization.CultureInfo.CurrentCulture.Calendar.GetWeekOfYear(date, System.Globalization.CalendarWeekRule.FirstFourDayWeek, DayOfWeek.Monday);

			return weekNr; ;
		}
        
        public static string GetStringDateTimeBg(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";

			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";

			string pattern = "dd.MM.yy HH:mm";

			return val.ToString(pattern);
		}

        public static string GetStringDateBg(object date)
        {
            if (date == System.DBNull.Value || date == null || !(date is DateTime))
                return "";

            DateTime val = (DateTime)date;
            if (val == ParseMethods.VALUE_BAD_DATE)
                return "";

            string pattern = "dd.MM.yyyy";

            return val.ToString(pattern);
        }

		public static string GetStringTime(object date)
		{
			if (date == System.DBNull.Value || date == null || !(date is DateTime))
				return "";

			DateTime val = (DateTime)date;
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";

			string pattern = "HH:mm";

			return val.ToString(pattern);
		}

		public static string GetStringInt(int val)
		{
			if (val == ParseMethods.VALUE_BAD_INT)
				return "";
			return val.ToString();
		}

		public static string GetDBStringDate(DateTime val)
		{
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";

			string pattern = "yyyyMMdd";
			return val.ToString(pattern);
		}

		public static string GetDBStringDateTime(DateTime val)
		{
			if (val == ParseMethods.VALUE_BAD_DATE)
				return "";

			string pattern = "yyyyMMdd HH:mm:ss.fff";
			return val.ToString(pattern);
		}

		public static string FormatDouble(object val, int prec, bool emptyIfBad)
		{
			double v = ParseMethods.ReadDouble(val);
			string frm = "";
			if (prec >= 0)
			{
				frm = "#.";
				for (int i = 0; i < prec; i++)
					frm += "#";
			}
			if (v == ParseMethods.VALUE_BAD_DOUBLE && emptyIfBad)
				return "";
			else
				return v.ToString(frm);
		}

        public static string GetPositiveDouble(object val, string format)
        {
            double d = ParseMethods.ReadDouble(val);
            if (d < 0)
                return "";
            return d.ToString(format);
        }

		public static string GetHourFormat(int minutes)
		{
			if (minutes == ParseMethods.VALUE_BAD_INT)
				return "";

			if (minutes < 0)
				minutes = -minutes;

			string strMin = ((minutes / 60).ToString("00") + ":" + (minutes % 60).ToString("00"));

			return strMin;
		}	
	}
}
