using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using Common;
using System.Data.SqlClient;

namespace DataLayer
{
	public class MSSqlQueries
	{
		#region Service Methods
		public static void AddParameter(SqlCommand command, string paramName, SqlDbType paramType, int paramSize, ParameterDirection paramDirection, object paramValue)
		{
			bool paramExists = false;
			try
			{
				if (command.Parameters[paramName] != null)
					paramExists = true;
			}
			catch
			{
				paramExists = false;
			}

			if (paramExists)
				return;

			SqlParameter param = command.CreateParameter();
			param.ParameterName = paramName;
			param.Direction = paramDirection;
			param.SqlDbType = paramType;
			if (paramSize > 0) param.Size = paramSize;
			if (paramValue != null) param.Value = paramValue;
			command.Parameters.Add(param);
		}
		#endregion

        public class Person
        {
            public static DataTable GetPersonalDetails(int personID, string password)
            {
                DataTable data = new DataTable();
                string sql = @"EXEC dbo.WebGetPersonalDetails @PersonID,@pwd";

                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    AddParameter(command, "@pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, password);
                    helper.Fill(data);
                }
                return data;
            }

            public static DataTable GetRequestDates(int personID)
            {
                DataTable data = new DataTable();
                string sql = @"EXEC dbo.WebGetRequestDates @PersonID";

                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    helper.Fill(data);
                }
                return data;
            }

            public static void FillResults(DataTable dtResults, ResultsFilter resultsFilter)
            {
                if (dtResults == null)
                    dtResults = new DataTable();

                StringBuilder builder = new StringBuilder();
                builder.Append(@"EXEC dbo.WebGetResults @PersonID,@pwd,@RequestDate,@ServiceID,@StartDate,@EndDate");

                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, resultsFilter.PersonID);
                    AddParameter(command, "@pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, resultsFilter.Password);

                    if (Common.Validation.IsValidDate(resultsFilter.RequstDate))
                        AddParameter(command, "@RequestDate", SqlDbType.DateTime, -1, ParameterDirection.Input, resultsFilter.RequstDate);
                    else
                        AddParameter(command, "@RequestDate", SqlDbType.DateTime, -1, ParameterDirection.Input, System.DBNull.Value);
                    
                    if (resultsFilter.ServiceID>0)
                        AddParameter(command, "@ServiceID", SqlDbType.Int, -1, ParameterDirection.Input, resultsFilter.ServiceID);
                    else
                        AddParameter(command, "@ServiceID", SqlDbType.Int, -1, ParameterDirection.Input, System.DBNull.Value);

                    if (Common.Validation.IsValidDate(resultsFilter.StartDate))
                        AddParameter(command, "@StartDate", SqlDbType.DateTime, -1, ParameterDirection.Input, resultsFilter.StartDate);
                    else
                        AddParameter(command, "@StartDate", SqlDbType.DateTime, -1, ParameterDirection.Input, DateTime.Today.AddYears(-100));

                    if (Common.Validation.IsValidDate(resultsFilter.EndDate))
                        AddParameter(command, "@EndDate", SqlDbType.DateTime, -1, ParameterDirection.Input, resultsFilter.EndDate);
                    else
                        AddParameter(command, "@EndDate", SqlDbType.DateTime, -1, ParameterDirection.Input, DateTime.Today.AddYears(100));

                    command.CommandText = builder.ToString();
                    helper.Fill(dtResults);
                }
            }

            public static DataTable GetServices(int personID)
            {
                DataTable data = new DataTable();
                string sql = @"EXEC dbo.WebGetServices @PersonID";

                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    helper.Fill(data);
                }
                return data;
            }

            public static int CreateUser(int personID, string password, DateTime startDate, DateTime endDate)
            {
                string sql = @"EXEC dbo.WebUsers_Ins @PersonID,@pwd,@StartDate,@EndDate";
                int result = 0;
                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    AddParameter(command, "@pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, password);
                    AddParameter(command, "@StartDate", SqlDbType.DateTime, -1, ParameterDirection.Input, startDate);
                    AddParameter(command, "@EndDate", SqlDbType.DateTime, -1, ParameterDirection.Input, endDate);
                    result = helper.ExecuteNonQuery(sql);
                }
                return result;
            }

            public static int ChangePassword(int webUserID, int personID, string oldPassword, string newPassword)
            {
                string sql = @"EXEC dbo.WebUsers_UpdPWD @webUserID,@PersonID,@old_pwd,@new_pwd";
                int result = 0;
                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;

                    if (webUserID>0)
                        AddParameter(command, "@webUserID", SqlDbType.Int, -1, ParameterDirection.Input, webUserID);
                    else
                        AddParameter(command, "@webUserID", SqlDbType.Int, -1, ParameterDirection.Input, System.DBNull.Value);
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    AddParameter(command, "@old_pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, oldPassword);
                    AddParameter(command, "@new_pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, newPassword);
                    result = helper.ExecuteNonQuery(sql);
                }
                return result;
            }

            /// <summary>
            /// Performs validation of user password.
            /// </summary>
            /// <param name="personID">User ID to check password</param>
            /// <param name="password">Password that must match the user ID</param>
            /// <returns>1 - if a match is found. 0 - if user and password pair does not match.</returns>
            public static DataTable CheckPassword(int personID, string password)
            {
                DataTable dataTable = new DataTable(DBS.T_WEBUSERS);
                string sql = @"SELECT * FROM WebUsers_CheckUpd(@PersonID,@new_pwd)";
                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.Text;
                    command.CommandText = sql;
                    AddParameter(command, "@PersonID", SqlDbType.Int, -1, ParameterDirection.Input, personID);
                    AddParameter(command, "@new_pwd", SqlDbType.VarChar, -1, ParameterDirection.Input, password);
                    helper.Fill(dataTable);
                }
                
                return dataTable;
            }
        }

        public class Hospital
        {
            public static DataTable GetHospitalDetails()
            {
                DataTable data = new DataTable();
                string sql = @"WebGetHospitalDetails";

                using (IHelper helper = MainDALC.GetHelper())
                {
                    SqlCommand command = (SqlCommand)helper.SelectCommand;
                    command.Parameters.Clear();
                    command.CommandType = CommandType.StoredProcedure;
                    command.CommandText = sql;
                    helper.Fill(data);
                }
                return data;
            }
        }
	}
}
