using System;
using System.IO;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Common;
using BusinessLayer;
using CrystalDecisions.CrystalReports.Engine;
using CrystalDecisions.Shared;

namespace GammaMultilabWeb
{
    public class ReportHelper
    {
        public static void BindReport(Reports.RequestResultsReport report, ResultsFilter filter)
        {
            UserData user = UserData.CurrentUser;

            SetReportText(report, "requestresultsreport");

            (report.ReportDefinition.ReportObjects["txtFirstName"] as TextObject).Text = user.FirstName;
            (report.ReportDefinition.ReportObjects["txtSecondName"] as TextObject).Text = user.SecondName;
            (report.ReportDefinition.ReportObjects["txtLastName"] as TextObject).Text = user.LastName;
            (report.ReportDefinition.ReportObjects["txtPersonalNr"] as TextObject).Text = user.EGN;
            (report.ReportDefinition.ReportObjects["txtAddress"] as TextObject).Text = user.Address;
            (report.ReportDefinition.ReportObjects["txtPhone"] as TextObject).Text = user.Phone;
            (report.ReportDefinition.ReportObjects["txtDate"] as TextObject).Text = FormatMethods.GetStringDateTimeBg(DateTime.Now);

            DS.DSRequestResult dsRequestResult = new DS.DSRequestResult();
                        
            PatoState ps;
            int patoID;
            string patoName;

            DataTable dtResult = BusinessHelper.Person.GetResults(filter, user);
            foreach (DataRow row in dtResult.Rows)
            {
                patoID = ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_PATO]);
                if (patoID >= 0)
                {
                    ps = (PatoState)ParseMethods.ParseEnum(typeof(PatoState), patoID);
                    patoName = Multilanguage.GetTranslatedString("requests.aspx", ps.ToString().ToLower());
                }
                else
                {
                    patoID = -1;
                    patoName = "";
                }

                dsRequestResult.Results.AddResultsRow(ParseMethods.ReadIntZero(row[DBS.F_LABEXAM_REQUEST_ID]),
                    FormatMethods.GetStringDateTimeBg(ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME])),
                    ParseMethods.ReadIntZero(row[DBS.F_LABTEST_SERVICE_ID]),
                    ParseMethods.ReadIntZero(row[DBS.F_LABTESTRESULT_ACTIVITY_ID]),
                    row[DBS.F_LABTESTRESULT_NAME].ToString(),
                    row[DBS.F_LABTESTRESULT_RESULT].ToString(),
                    row[DBS.F_LABTESTRESULT_MUNIT].ToString(),
                    row[DBS.F_LABTESTRESULT_LOW_LIMIT].ToString(),
                    row[DBS.F_LABTESTRESULT_HIGH_LIMIT].ToString(),
                    patoID,
                    patoName,
                    ParseMethods.ReadIntZero(row[DBS.F_LEVEL]));
            }

            string filePath = HttpContext.Current.Server.MapPath("~/img/report_logo.jpg");
            byte[] imgLogo = FileHelper.GetFile(filePath);
            FillLogo(dsRequestResult.Image.LogoColumn, imgLogo);

            report.SetDataSource(dsRequestResult);            
            report.Subreports["Logo"].SetDataSource(dsRequestResult);
        }

        private static void FillLogo(DataColumn colImage, byte[] imgLogo)
        {
            if (colImage==null || colImage.DataType!= typeof(byte[]) || imgLogo == null || imgLogo.Length == 0)
                return;

            DataTable dtImage = colImage.Table;
            dtImage.Rows.Clear();

            DataRow row = dtImage.NewRow();
            row[colImage] = imgLogo;
            row.Table.Rows.Add(row);
        }

        public static void SetReportText(ReportDocument report, string key)
        {
            Hashtable reportData = Multilanguage.GetGroupStrings(key);
            if (reportData != null)
            {
                if (reportData.Contains("Title"))
                    report.SummaryInfo.ReportTitle = reportData["Title"].ToString();
                ReportObject repObj;
                TextObject textObj;
                FieldHeadingObject headObj;
                for (int i = 0; i < report.ReportDefinition.ReportObjects.Count; i++)
                {
                    repObj = report.ReportDefinition.ReportObjects[i];
                    switch (repObj.Kind)
                    {
                        case ReportObjectKind.TextObject:
                            textObj = (TextObject)repObj;
                            if (reportData.Contains(textObj.Name))
                                textObj.Text = (string)reportData[textObj.Name];
                            break;
                        case ReportObjectKind.FieldHeadingObject:
                            headObj = (FieldHeadingObject)repObj;
                            if (reportData.Contains(headObj.Name))
                                headObj.Text = (string)reportData[headObj.Name];
                            break;
                        case ReportObjectKind.SubreportObject:
                            SubreportObject subRep = (SubreportObject)repObj;
                            ReportDocument subReport = report.OpenSubreport(subRep.SubreportName);
                            SetReportText(subReport, subRep.SubreportName.Replace(".rpt", ""));
                            break;
                    }
                }
            }
        }
    }
}
