using System;
using System.Data;
using System.Web.UI.WebControls;
using System.Collections;
using System.Globalization;

using Common;

namespace FrameWork
{
    public class UIHelper
    {
        public static void BindCombo(DropDownList control, DataView dataSource, string textField, string valueField, bool addEmptyItem, string selectedValue, string emptyItemText)
        {
            if (dataSource != null)
            {
                control.DataSource = dataSource;
                control.DataTextField = textField;
                control.DataValueField = valueField;
                control.DataBind();
            }
            if (addEmptyItem) control.Items.Insert(0, new ListItem(emptyItemText, ""));
            if (selectedValue != "")
            {
                ListItem item = control.Items.FindByValue(selectedValue);
                if (item == null)
                    return;
                control.SelectedValue = selectedValue;
            }
        }

        public static void BindCombo(DropDownList control, object dataSource, string textField, string valueField, bool addEmptyItem, string selectedValue, string emptyItemText)
        {
            control.DataSource = dataSource;
            control.DataTextField = textField;
            control.DataValueField = valueField;
            control.DataBind();
            if (addEmptyItem) control.Items.Insert(0, new ListItem(emptyItemText, ""));
            if (selectedValue != "" && control.Items.FindByValue(selectedValue) != null)
            {
                control.SelectedValue = selectedValue;
            }
        }

        public static bool SetComboValue(DropDownList cbo, string val)
        {
            if (val == null)
                return false;
            ListItem li = cbo.Items.FindByValue(val);
            if (li == null)
            {
                if (cbo.Items.Count > 0)
                    cbo.SelectedIndex = 0;
                return false;
            }

            cbo.SelectedValue = val;
            return true;
        }

        public static bool SetComboText(DropDownList cbo, string val)
        {
            if (val == null)
                return false;
            ListItem li = cbo.Items.FindByText(val);
            if (li == null)
            {
                if (cbo.Items.Count > 0)
                    cbo.SelectedIndex = 0;
                return false;
            }

            cbo.SelectedValue = li.Value;
            return true;
        }

        public static string GetComboValue(DropDownList cbo)
        {
            if (cbo.SelectedIndex < 0)
                return null;
            return cbo.SelectedValue;
        }

        public static string GetComboText(DropDownList cbo)
        {
            if (cbo.SelectedIndex < 0)
                return null;
            return cbo.SelectedItem.Text;
        }
        public static void BindGrid(DataGrid dgr, DataView source)
        {
            int idx = (int)Math.Ceiling(source.Table.Rows.Count / (float)dgr.PageSize);
            idx = dgr.CurrentPageIndex > idx - 1 ? idx - 1 : dgr.CurrentPageIndex;
            if (idx < 0) idx = 0;

            dgr.CurrentPageIndex = idx;
            dgr.DataSource = source;
            dgr.DataBind();
        }

        public static int GetRowIdx(DataTable src, object[] pkValues)
        {
            if (pkValues == null || pkValues.Length == 0 || pkValues[0] == null)
                return -1;
            if (src.PrimaryKey == null)
                return -1;

            DataRow goal = src.Rows.Find(pkValues);
            if (goal == null) return -1;
            for (int i = 0; i < src.Rows.Count; i++)
                if (src.Rows[i] == goal)
                    return i;
            return -1;
        }

        public static void BindGrid(DataGrid dgr, DataTable source)
        {
            int idx = (int)Math.Ceiling(source.Rows.Count / (float)dgr.PageSize);
            idx = dgr.CurrentPageIndex > idx - 1 ? idx - 1 : dgr.CurrentPageIndex;
            if (idx < 0) idx = 0;

            dgr.CurrentPageIndex = idx;
            dgr.DataSource = source;
            dgr.DataBind();
        }

        public static void ClearCombo(DropDownList combo)
        {
            combo.Items.Clear();
            combo.Items.Insert(0, "");
        }

        public static void SetLabel(Label lbl, object val)
        {
            if (val == null || val == System.DBNull.Value)
                lbl.Text = "";
            else if (val is string)
                lbl.Text = (string)val;
            else if (val is DateTime)
                lbl.Text = FormatMethods.GetStringDateBg(val);
            else
                lbl.Text = val.ToString();
        }

        public static void InitComboYears(DropDownList cboYear, int selectedYear)
        {
            int minYear = DateTime.Today.Year - 2;
            int maxYear = DateTime.Today.Year + 1;
            if (selectedYear > maxYear) maxYear = selectedYear;
            if (selectedYear < minYear) minYear = selectedYear;

            cboYear.Items.Clear();
            for (int i = minYear; i <= maxYear; i++)
                cboYear.Items.Add(new ListItem(i.ToString(), i.ToString()));
            cboYear.SelectedValue = selectedYear.ToString();
        }

        public static void InitComboMonths(DropDownList cboMonth, int selectedMonth)
        {
            cboMonth.Items.Clear();
            for (int i = 1; i <= 12; i++)
                cboMonth.Items.Add(new ListItem(i.ToString(), i.ToString()));
            cboMonth.SelectedValue = selectedMonth.ToString();
        }

        public static void InitComboWeeks(int year, DropDownList cboWeek, int selectedWeek)
        {
            cboWeek.DataValueField = "_AutoNum";
            cboWeek.DataTextField = "_AutoNum";
            cboWeek.DataSource = UIHelper.GetWeeks(year);
            cboWeek.DataBind();
            if (cboWeek.Items.FindByValue(selectedWeek.ToString()) != null)
                cboWeek.SelectedValue = selectedWeek.ToString();
        }

        public static DataTable GetWeeks(int year)
        {
            DataTable dtRes = new DataTable("Weeks");
            dtRes.Columns.Add("_AutoNum", typeof(int));
            int weeks = GetWeekNumber(new DateTime(year, 12, 31));

            for (int i = 1; i <= weeks; i++)
                dtRes.Rows.Add(new object[] { i });

            return dtRes;
        }

        public static int GetWeekNumber(DateTime date)
        {
            CultureInfo culture = CultureInfo.CurrentCulture;
            int week = culture.Calendar.GetWeekOfYear(date, culture.DateTimeFormat.CalendarWeekRule, culture.DateTimeFormat.FirstDayOfWeek);
            return week;
        }

        public static void SelectPreviousComboItem(DropDownList cbo, bool addNew)
        {
            if (cbo.SelectedIndex > 0)
                cbo.SelectedIndex--;
            else if (addNew)
            {
                int newItem = ParseMethods.ReadInt(cbo.SelectedValue) - 1;
                cbo.Items.Insert(0, new ListItem(newItem.ToString(), newItem.ToString()));
                cbo.SelectedIndex = 0;
            }
            else
                cbo.SelectedIndex = cbo.Items.Count - 1;
        }

        public static void SelectNextComboItem(DropDownList cbo, bool addNew)
        {
            if (cbo.SelectedIndex < cbo.Items.Count - 1)
                cbo.SelectedIndex++;
            else if (addNew)
            {
                int newItem = ParseMethods.ReadInt(cbo.SelectedValue) + 1;
                cbo.Items.Add(new ListItem(newItem.ToString(), newItem.ToString()));
                cbo.SelectedIndex = cbo.Items.Count - 1;
            }
            else
                cbo.SelectedIndex = 0;
        }

        public static void GetWeekStartEndDates(int year, int week, ref DateTime dateFrom, ref DateTime dateTo)
        {
            dateFrom = new DateTime(year, 1, 1);
            int currentWeek = GetWeekNumber(dateFrom);

            while (currentWeek != 1)
            {
                dateFrom = dateFrom.AddDays(1);
                currentWeek = GetWeekNumber(dateFrom);
            }

            while (currentWeek != week)
            {
                dateFrom = dateFrom.AddDays(1);
                currentWeek = GetWeekNumber(dateFrom);
            }

            dateTo = dateFrom.AddDays(6 - (int)dateFrom.DayOfWeek + (int)CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek);
        }
    }
}