using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Common;
using FrameWork;
using BusinessLayer;

namespace GammaMultilabWeb
{
    public partial class ChangePassword : FrameWork.PageBase
    {
        #region Initialization
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected override void LoadStaticData()
        {
            base.LoadStaticData();
            DataTable dtRequestDates = CurrentUserData.GetWebUserInfoTable();
            UIHelper.BindCombo(cboPasswordDate, dtRequestDates, DBS.F_NAME, DBS.F_WEBUSERS_WEBUSERID, true, null, Multilanguage.GetTranslatedString(PageFileName, "all"));

            lblWelcomeText.Text = Multilanguage.GetTranslatedString(PageFileName, "WelcomeText");
        }
        #endregion

        #region Event Handlers
        protected void btnSave_Click(object sender, EventArgs e)
        {
            Page.Validate();

            int webUserID = ParseMethods.ReadInt(cboPasswordDate.SelectedValue);
            string oldPassword = txtOldPassword.Text.Trim();
            string newPassword = txtNewPassword.Text.Trim();
            string confirmPassword = txtConfirmPassword.Text.Trim();

            if (oldPassword=="")
                ShowSingleError(lblOldPassword,txtOldPassword,StringConstants.ERROR_REQUIRED_FIELD);
            else if (webUserID>0)
            {
                WebUserInfo wi = CurrentUserData.WebUserInfo[webUserID];
                if (wi!=null && oldPassword!=wi.Password)
                   ShowSingleError(lblOldPassword, txtOldPassword, StringConstants.ERROR_PASSWORD_WRONG);
            }
                
            if (newPassword=="")
                ShowSingleError(lblNewPassword,txtNewPassword,StringConstants.ERROR_REQUIRED_FIELD);
            if (confirmPassword=="")
                ShowSingleError(lblConfirmPassword,txtConfirmPassword,StringConstants.ERROR_REQUIRED_FIELD);
            
            if (confirmPassword != newPassword)
                ShowSingleError(lblNewPassword, txtNewPassword, StringConstants.ERROR_PASSWORD_MISMATCH);
            else if (newPassword.Length < Administration.MinPasswordLength)
            {
                string errMsg = GetTranslatedError(StringConstants.ERROR_PASSWORD_LENGTH).Replace("%1",Administration.MinPasswordLength.ToString());
                ShowSingleError(lblNewPassword, txtNewPassword, errMsg);
            }
                
            if (Page.IsValid)
            {
                bool result = CurrentUserData.ChangePassword(webUserID, oldPassword, newPassword);
                if (result)
                {
                    ShowGeneralError(StringConstants.MSG_PASSWORD_CHANGE_SUCCESS, lblNotification);
                    LoadStaticData();
                }
                else
                    ShowGeneralError(StringConstants.ERROR_PASSWORD_CHANGE_FAILED);
            }
        }
        #endregion

        #region Methods
        protected override void ClearControlStyles()
        {
            ClearControlStyle(lblOldPassword);
            ClearControlStyle(lblNewPassword);
            ClearControlStyle(lblConfirmPassword);

            ClearControlStyle(txtOldPassword);
            ClearControlStyle(txtNewPassword);
            ClearControlStyle(txtConfirmPassword);

        }
        #endregion
    }
}
