using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;

using Common;

namespace GammaMultilabWeb
{
    public class Global : System.Web.HttpApplication
    {
        private static Trace trace = null;

        public static System.Web.UI.Page InstanceClassifier
        {
            set
            {
                if (Trace.TracingEnabled)
                    trace.SetInstanceClassifier(value, value.Request); ;
            }
        }

        void Application_Start(object sender, EventArgs e)
        {
            Common.ApplicationConfiguration.OnApplicationStart(Context.Server.MapPath(Context.Request.ApplicationPath));

            if (Trace.TracingEnabled)
                trace = Trace.ApplicationStart();

            string langFile = Server.MapPath(System.Configuration.ConfigurationManager.AppSettings[Common.StringConstants.APPLICATION_KEY_LANGUAGE_DATA].ToString());
            Application.Add(Common.StringConstants.APPLICATION_KEY_LANGUAGE_DATA, FrameWork.PageBase.LoadLanguageData(langFile));
        }

        void Application_End(object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
                trace.ApplicationEnd();

        }

        void Application_Error(object sender, EventArgs e)
        {
            Exception ex = Server.GetLastError();

            if (ex is HttpUnhandledException && ex.InnerException != null)
                ex = ex.InnerException;

            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            builder.Append("Unhandled Exception:\r\n");
            Common.ApplicationLog.WriteError(Common.ApplicationLog.FormatException(ex, builder.ToString()), Common.ApplicationConfiguration.TracingTraceFile);

            if (Trace.TracingEnabled)
                trace.ApplicationError(Request, ex);

            Response.Redirect("~/ErrorPage.aspx");

        }

        void Application_AuthenticateRequest(object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
                trace.ApplicationAuthenticateRequest();

            System.Web.HttpContext.Current.User = null;
            HttpCookie sessionCookie = System.Web.HttpContext.Current.Request.Cookies["ASP.NET_SessionID"];
            if (sessionCookie == null)
                return;

            string cookieName = sessionCookie.Value;

            HttpCookie authCookie = System.Web.HttpContext.Current.Request.Cookies[cookieName];
            if (authCookie == null || authCookie.Value == "")// There is no authentication cookie.
                return;

            FormsAuthenticationTicket authTicket = null;
            try
            {
                authTicket = FormsAuthentication.Decrypt(authCookie.Value);
                if (authTicket.Expired)
                    return;
            }
            catch (Exception ex)
            {
                Common.ApplicationLog.LogException(ex, "Handled exception: ");
                return;
            }

            if (null == authTicket)// Cookie failed to decrypt.
                return;


            string[] roles = authTicket.UserData.Split(new char[] { '|' });

            FormsIdentity id = new FormsIdentity(authTicket);

            System.Security.Principal.GenericPrincipal principal = new System.Security.Principal.GenericPrincipal(id, roles);

            System.Web.HttpContext.Current.User = principal;

            BusinessLayer.Administration.RenewAuthenticationTicket(authTicket.Name, sessionCookie);
        }

        void Session_Start(object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
                trace.SessionStart(Session, Request);	
        }

        void Session_End(object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
                trace.SessionEnd(Session);
        }

        protected void Application_EndRequest(Object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
                trace.ApplicationEndRequest(Request, Response);
        }

        protected void Application_BeginRequest(Object sender, EventArgs e)
        {
            if (Trace.TracingEnabled)
				trace.ApplicationBeginRequest(Request);
		}
    }
}