using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Common;
using BusinessLayer;

namespace GammaMultilabWeb
{
    public partial class Home : FrameWork.PageBase
    {
        #region Fields
        private string SortExp
        { 
            get
            {
                object sortExp = ViewState[StringConstants.VIEWSTATE_KEY_SORT];
                if (sortExp == null)
                    sortExp = DBS.F_REQUEST_REQUEST_DATE_TIME;

                return sortExp.ToString();
            }
            set
            {
                if (value != null)
                {
                    object sortExp = SortExp;
                    sortExp = sortExp.ToString() == value ? sortExp + " DESC" : value;
                    ViewState[StringConstants.VIEWSTATE_KEY_SORT] = sortExp;
                }
            }
        }
        #endregion
        #region Initialization
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
                BindGrid();
        }

        private void BindGrid()
        {
            DataTable dtRequestDates = BusinessHelper.Person.GetRequestDatesWithAuthorization(CurrentUserData);
            dtRequestDates.DefaultView.Sort = SortExp;

            gvRequestDates.ShowFooter = gvRequestDates.EditIndex < 0 
                && dtRequestDates.Select(DBS.F_AUTHORIZED+"=false").Length!=0;
            gvRequestDates.DataSource = dtRequestDates;
            gvRequestDates.DataBind();
        }

        protected override void LoadStaticData()
        {
            base.LoadStaticData();

            SetControlText(lblFirstName, CurrentUserData.FirstName);
            SetControlText(lblSecondName, CurrentUserData.SecondName);
            SetControlText(lblFamilyName, CurrentUserData.LastName);
            SetControlText(lblPersonalNr, CurrentUserData.EGN);
            SetControlText(lblPhone, CurrentUserData.Phone);
            SetControlText(lblAddress, CurrentUserData.Address);

            lblWelcomeText.Text = Multilanguage.GetTranslatedString(PageFileName, "WelcomeText");
        }
        #endregion

        #region Methods
        protected override string GetFieldEmptyText()
        {
            string emptyText = Multilanguage.GetTranslatedString(PageFileName, "nodata");
            if (emptyText==StringConstants.NOT_TRANSLATED)
                return base.GetFieldEmptyText();
            return emptyText;
        }
        #endregion

        #region Event Handlers
        protected void gvRequestDates_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            TextBox txtPass = null;
            switch (e.Row.RowType)
            { 
                case DataControlRowType.DataRow:
                    DataRow row = (e.Row.DataItem as DataRowView).Row;
                    bool auth = ParseMethods.ReadBool(row[DBS.F_AUTHORIZED]);
                    Image img = e.Row.FindControl("imgAuth") as Image;
                    img.ImageUrl = auth ? "~/Img/auth.gif" : "~/Img/notauth.gif";

                    Label lblDate = e.Row.FindControl("lblDate") as Label;
                    DateTime requestDate = ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                    lblDate.Text = FormatMethods.GetStringDateBg(requestDate);

                    LinkButton lnk;

                    if (e.Row.DataItemIndex != gvRequestDates.EditIndex)
                    {
                        lnk = e.Row.FindControl("lnkLogin") as LinkButton;
                        lnk.Visible = !auth;

                        lnk = e.Row.FindControl("lnkDelete") as LinkButton;
                        lnk.CommandArgument = requestDate.Ticks.ToString();
                        lnk.Visible = auth;

                        lnk = e.Row.FindControl("lnkDelete") as LinkButton;
                        lnk.OnClientClick = "JavaScript: return confirm('" + Multilanguage.GetTranslatedString(PageFileName, "confirmdelete") + "');";
                    }
                    else 
                    { 
                        lnk = e.Row.FindControl("lnkAuth") as LinkButton;
                        lnk.CommandArgument = requestDate.Ticks.ToString();
                        txtPass = e.Row.FindControl("txtPassword") as TextBox;
                    }
                    break;
                case DataControlRowType.Header:
                    lnk = e.Row.FindControl("lnkHeaderDate") as LinkButton;
                    lnk.CommandArgument = DBS.F_REQUEST_REQUEST_DATE_TIME;
                    break;
                case DataControlRowType.Footer:
                    txtPass = e.Row.FindControl("txtPassword") as TextBox;
                    break;
            }

            if (txtPass != null)
            {
                string script = "FocusControl('" + txtPass.ClientID + "');";
                if (!Page.ClientScript.IsStartupScriptRegistered("focus"))
                    ClientScript.RegisterStartupScript(typeof(string), "focus", script, true);
            }
        }

        protected void gvRequestDates_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
        {
            gvRequestDates.EditIndex = -1;
            BindGrid();
        }

        protected void gvRequestDates_RowEditing(object sender, GridViewEditEventArgs e)
        {
            gvRequestDates.EditIndex = e.NewEditIndex;
            BindGrid();
        }

        protected void gvRequestDates_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            switch (e.CommandName.ToLower())
            {
                case "auth":
                    TextBox txtPassword = (e.CommandSource as LinkButton).Parent.Parent.FindControl("txtPassword") as TextBox;
                    ClearControlStyle(txtPassword);

                    long ticks = ParseMethods.ReadLongZero(e.CommandArgument);

                    DateTime requestDate = ticks > 0 ? new DateTime(ticks) : ParseMethods.VALUE_BAD_DATE;
                    string password = txtPassword.Text.Trim();
                    if (password == "")
                        ShowSingleError(null, txtPassword, StringConstants.ERROR_REQUIRED_FIELD);
                    else if (CurrentUserData.AuthorizeDate(requestDate, password))
                    {
                        gvRequestDates.EditIndex = -1;
                        BindGrid();
                    }
                    else
                        ShowSingleError(null, txtPassword, StringConstants.ERROR_PASSWORD_DATE_MISMATCH);
                    break;
                case "sort":
                    SortExp = e.CommandArgument.ToString();
                    BindGrid();
                    break;
                case "remove":
                    ticks = ParseMethods.ReadLongZero(e.CommandArgument);
                    if (ticks <= 0)
                        return;
                    requestDate = new DateTime(ticks);
                    CurrentUserData.UnAuthorizeDate(requestDate);
                    BindGrid();
                    break;
            }
        }
        #endregion

        
    }
}
