using System;
using System.Drawing;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using ZedGraph;
using Common;
using BusinessLayer;

namespace GammaMultilabWeb.Reports
{
    public partial class ActivityChart : FrameWork.ChartBase
    {
        #region Fields
        private DataTable _dtResultRequestDates;
        private DataTable _dtResult;

        private ArrayList _activityIDs;
        private ArrayList ActivityIDs
        { 
            get
            {
                if (_activityIDs == null)
                    _activityIDs = CurrentUserData.PageCache["activityIDs"] as ArrayList;
                return _activityIDs;
            }
        }

        private ResultsFilter _resultsFilter;
        private ResultsFilter ResultsFilter
        {
            get
            {
                if (_resultsFilter == null)
                    _resultsFilter = CurrentUserData.PageCache["resultFilter"] as ResultsFilter;
                return _resultsFilter;
            }
        }

        private string[] _colors;
        private string[] Colors
        {
            get 
            {
                if (_colors == null)
                {
                    string strColors = Multilanguage.GetTranslatedString(StringConstants.MESSAGES, "colors");
                    if (strColors == StringConstants.NOT_TRANSLATED)
                        strColors = "#000000";
                    _colors = strColors.Split(new char[] { ';' });
                }
                return _colors;
            }
        }
        #endregion

        #region Initialization
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
        }

        protected override void InitPageCache()
        {
            //Override in order not to clear the page cache
        }
        #endregion

        #region Methods
        protected override void RenderGraph(System.Drawing.Graphics g, ZedGraph.MasterPane masterPane)
        {
            DataTable results = GetResults();
            DataTable requestDates = GetResultRequestDates(results);

            if (results.Rows.Count == 0 || requestDates.Rows.Count == 0)
                return;

            base.RenderGraph(g, masterPane);

            DateTime []arrRequestDates = new DateTime[requestDates.Rows.Count];
            string []strRequestDates = new string[requestDates.Rows.Count];
            for (int i = 0; i < requestDates.Rows.Count; i++)
            {
                arrRequestDates[i] = ParseMethods.ReadDate(requestDates.Rows[i][DBS.F_REQUEST_REQUEST_DATE_TIME]);
                strRequestDates[i] = FormatMethods.GetStringDateBg(arrRequestDates[i]);
            }

            GraphPane pane = masterPane[0];
            pane.XAxis.TextLabels = strRequestDates;
           
            LineItem curve;
            string[] colors = Colors;
            double[] arrActivityValues = new double[arrRequestDates.Length];
            double[] arrRequestDateValues = new double[arrRequestDates.Length];
            System.Drawing.Color activityColor;
            
            string activityName;
            DataRow resultRow = null;
            
            int activityID;
            DateTime requestDate;
            Fill symbolFill = new Fill(Color.White);
            for (int i = 0; i < ActivityIDs.Count; i++)
            {
                activityName = "";
                activityID = ParseMethods.ReadInt(ActivityIDs[i]);
                for (int j= 0;j<arrRequestDates.Length;j++)
                {
                    requestDate = arrRequestDates[j];
                    resultRow = results.Rows.Find(new object[]{activityID, requestDate});
                    if (resultRow==null)
                    {
                        arrActivityValues[j] = double.NaN;
                        arrRequestDateValues[j] = double.NaN;
                        continue;
                    }
                    else if (activityName=="")//TODO: add activity unit
                        activityName = resultRow[DBS.F_LABTESTRESULT_NAME].ToString();

                    arrActivityValues[j] = ParseMethods.ReadFloatZero(resultRow[DBS.F_LABTESTRESULT_RESULT]);
                    arrRequestDateValues[j] = j+1;
                }
                
                activityColor = ParseMethods.ReadColor(Colors[i]);

                curve = pane.AddCurve(activityName, arrRequestDateValues, arrActivityValues, activityColor, SymbolType.Circle);
                curve.Line.Width = 1.5f;
                curve.Symbol.Fill = symbolFill;
                curve.Symbol.Size = 5;
            }
            
            pane.AxisChange(g);		
        }


        private DataTable GetResults()
        {
            if (_dtResult == null)
            {
                _dtResult = BusinessHelper.Person.GetResults(ResultsFilter, CurrentUserData);
                int activityID;
                DataRow row;
                for (int i=0;i<_dtResult.Rows.Count;)
                {
                    row = _dtResult.Rows[i];
                    activityID = ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_ACTIVITY_ID]);
                    if (!ActivityIDs.Contains(activityID))
                        _dtResult.Rows.Remove(row);
                    else
                        i++;
                }
                _dtResult.PrimaryKey = new DataColumn[] { _dtResult.Columns[DBS.F_LABTESTRESULT_ACTIVITY_ID], _dtResult.Columns[DBS.F_REQUEST_REQUEST_DATE_TIME] };
            }
            return _dtResult;
        }

        private DataTable GetResultRequestDates(DataTable dtRequestResults)
        {
            if (_dtResultRequestDates == null)
                _dtResultRequestDates = BusinessHelper.Person.GetResultRequestDates(dtRequestResults);
            return _dtResultRequestDates;
        }
        #endregion
    }
}
