using System;
using System.Drawing;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Common;
using ZedGraph;

namespace FrameWork
{
    public partial class ChartBase : FrameWork.PageBase
    {
        #region Fields
        protected ChartMaster MasterPage
        {
            get { return this.Master as ChartMaster; }
        }

        private ZedGraphWeb graph;
        public ZedGraphWeb Graph
        {
            get
            {
                if (graph == null && MasterPage != null)
                    graph = MasterPage.Graph;
                return graph;
            }
        }

        private System.Drawing.Graphics g;
        private ZedGraph.MasterPane masterPane;
        //int []arrWeekNr = null;
        //double []arrPPCount = null;
        //double []arrPPAvailable = null;		
        //string []weekNr = null;
        //double []arrPPEngaged = null;
        //double []arrPPDiff = null;

        #endregion

        #region Initialization
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (MasterPage != null)
                MasterPage.RenderGraph += new ZedGraphWebControlEventHandler(MasterPage_RenderGraph);
        }

        void MasterPage_RenderGraph(Graphics g, MasterPane pane)
        {
            this.g = g;
            this.masterPane = pane;
            RenderGraph(g, pane);
        }

        protected virtual void RenderGraph(Graphics g, MasterPane masterPane)
        {
            GraphPane pane = masterPane[0];

            pane.BarBase = BarBase.X;
            pane.BarType = BarType.ClusterHiLow;
            pane.MarginLeft = 30;
            pane.Title = Multilanguage.GetTranslatedString(PageFileName, "GraphTitle"); ;
            pane.PaneBorder.IsVisible = false;
            Color clr = ParseMethods.ReadColor("#75B8FE");
            pane.AxisFill = new Fill(Color.White, clr, 45.0F);

            pane.YAxis.Title = Multilanguage.GetTranslatedString(PageFileName, "YAxisTitle");
            pane.YAxis.IsMinorTic = false;
            pane.YAxis.IsOppositeTic = false;
            pane.YAxis.IsMinorOppositeTic = false;
            pane.YAxis.IsMinorInsideTic = false;
            pane.YAxis.IsMinorOppositeTic = false;
            pane.YAxis.IsShowGrid = true;
            pane.YAxis.TicSize = 3;

            pane.XAxis.Title = Multilanguage.GetTranslatedString(PageFileName, "XAxisTitle");
            pane.XAxis.IsMinorTic = false;
            pane.XAxis.IsMinorOppositeTic = false;
            pane.XAxis.IsOppositeTic = false;
            pane.XAxis.Type = AxisType.Text;
            pane.XAxis.IsPreventLabelOverlap = true;

            pane.Legend.IsHStack = false;
            pane.Legend.Position = LegendPos.Right;
            pane.Legend.FontSpec.Size = 12;
            //pane.Legend.IsVisible = false;

        }
        #endregion

        #region Help Methods
        public static void EntitleBars(GraphPane pane)
        {
            string lab;
            float fontSize, inc, barWidth, coef = 0.02f; ;
            bool baseX = pane.BarBase == BarBase.X;
            ArrayList arrValues, arrayBars = new ArrayList();
            TextItem text;

            foreach (CurveItem c in pane.CurveList)
            {
                if (!(c is BarItem)) continue;

                arrValues = new ArrayList();
                arrayBars.Add(arrValues);

                for (int i = 0; i < c.Points.Count; i++)
                    arrValues.Add(baseX ? c.Points[i].Y : c.Points[i].X);
            }

            barWidth = (1 - pane.MinBarGap * (arrayBars.Count + 1)) / arrayBars.Count;
            inc = barWidth + pane.MinBarGap;
            fontSize = 14.5f - coef * (arrayBars[0] as ArrayList).Count / barWidth;

            if (fontSize < 7)
                return;

            float[] shift = new float[arrayBars.Count];
            for (int i = 0; i < arrayBars.Count; i++)
            {
                if (i == 0)
                    shift[i] = pane.MinBarGap + barWidth / 2 - 0.5f;
                else
                    shift[i] = shift[i - 1] + inc;
            }

            ArrayList arr;

            for (int j = 0; j < arrayBars.Count; j++)
            {
                arr = arrayBars[j] as ArrayList;
                for (int i = 0; i < arr.Count; i++)
                {
                    lab = " " + ((double)arr[i]).ToString("F");
                    text = new TextItem(lab, (float)(i + 1) + shift[j], (float)((double)arr[i] < 0 ? 0.0 : (double)arr[i]));

                    text.Location.CoordinateFrame = CoordType.AxisXYScale;
                    text.Location.AlignH = AlignH.Left;
                    text.Location.AlignV = AlignV.Center;
                    text.FontSpec.Border.IsVisible = false;
                    text.FontSpec.Fill.IsVisible = false;
                    text.FontSpec.Angle = 90;
                    text.FontSpec.Size = fontSize;
                    pane.GraphItemList.Add(text);
                }
            }
        }

        #endregion
    }
}
