using System;
using System.IO;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CrystalDecisions.CrystalReports;
using CrystalDecisions.CrystalReports.Engine;
using CrystalDecisions.Shared;

using Common;
using FrameWork;
using BusinessLayer;

namespace GammaMultilabWeb
{
    public partial class Requests : FrameWork.PageBase
    {
        #region Fields
        private DataTable _dtResults = null;        
        private string SortExpDtl
        {
            get 
            { 
                object sortExp = ViewState["SortExpDtl"];
                if (sortExp != null)
                    return sortExp.ToString();
                return null;
            }
            set 
            {
                string sortExp = SortExpDtl;
                sortExp = value == sortExp ? value + " DESC" : value;
                ViewState["SortExpDtl"] = sortExp;
            }
        }

        private Reports.RequestResultsReport requestResultsReport = new Reports.RequestResultsReport();
        
        #endregion

        #region Initialization
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected override void LoadStaticData()
        {
            base.LoadStaticData();
            DataTable dtServices = BusinessHelper.Person.GetServices(CurrentUserData.UserID);
            UIHelper.BindCombo(cboService, dtServices, DBS.F_SERVICE_NAME, DBS.F_SERVICE_ID, true, "", Multilanguage.GetTranslatedString(PageFileName, "allservices"));

            DataTable dtRequestDates = GetRequestDatesWithAuthorization(CurrentUserData);
            UIHelper.BindCombo(cboRequestDate, dtRequestDates, DBS.F_NAME, DBS.F_ID, true, "", Multilanguage.GetTranslatedString(PageFileName, "allrequests"));
        }
        #endregion

        #region Event Handlers

        protected void btnShow_Click(object sender, EventArgs e)
        {
            BindList();
        }

        protected void gv_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            switch (e.Row.RowType)
            { 
                case DataControlRowType.DataRow:

                    DataRow row = (e.Row.DataItem as DataRowView).Row;
                    Label lbl = e.Row.FindControl("lblName") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_NAME].ToString());

                    lbl = e.Row.FindControl("lblResult") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_RESULT].ToString());

                    lbl = e.Row.FindControl("lblU") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_MUNIT].ToString());

                    lbl = e.Row.FindControl("lblLL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_LOW_LIMIT].ToString());

                    lbl = e.Row.FindControl("lblUL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_HIGH_LIMIT].ToString());

                    lbl = e.Row.FindControl("lblPato") as Label;
                    PatoState ps = (PatoState)ParseMethods.ParseEnum(typeof(PatoState), ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_PATO]));
                    string patoState = Multilanguage.GetTranslatedString(PageFileName, ps.ToString().ToLower());
                    SetControlText(lbl, patoState);

                    lbl = e.Row.FindControl("lblDate") as Label;
                    SetControlText(lbl, FormatMethods.GetStringDateBg(row[DBS.F_REQUEST_REQUEST_DATE_TIME]));

                    break;
            }
        }

        protected void dtl_ItemDataBound(object sender, DataListItemEventArgs e)
        {
            switch (e.Item.ItemType)
            { 
                case ListItemType.Item:
                case ListItemType.AlternatingItem:
                    DataRow row = (e.Item.DataItem as DataRowView).Row;
                    Label lbl = e.Item.FindControl("lblName") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_NAME].ToString());

                    lbl = e.Item.FindControl("lblResult") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_RESULT].ToString());

                    lbl = e.Item.FindControl("lblU") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_MUNIT].ToString());

                    lbl = e.Item.FindControl("lblLL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_LOW_LIMIT].ToString());

                    lbl = e.Item.FindControl("lblUL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_HIGH_LIMIT].ToString());

                    lbl = e.Item.FindControl("lblPato") as Label;
                    string patoState = "-";
                    PatoState ps = (PatoState)ParseMethods.ParseEnum(typeof(PatoState), ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_PATO]));
                    if (ps != PatoState.None)
                        patoState = Multilanguage.GetTranslatedString(PageFileName, ps.ToString().ToLower());

                    SetControlText(lbl, patoState);

                    lbl = e.Item.FindControl("lblDate") as Label;
                    SetControlText(lbl, FormatMethods.GetStringDateBg(row[DBS.F_REQUEST_REQUEST_DATE_TIME]));

                    int requestID = ParseMethods.ReadInt(row[DBS.F_LABEXAM_REQUEST_ID]);
                    int serviceID = ParseMethods.ReadInt(row[DBS.F_LABTEST_SERVICE_ID]);
                    
                    GridView gv = e.Item.FindControl("gv") as GridView;
                    gv.Visible = false;
                    BindGrid(gv, requestID, serviceID);

                    ImageButton imgBtn = e.Item.FindControl("imgBtn") as ImageButton;
                    imgBtn.Visible = gv.Rows.Count > 0;
                    imgBtn.ImageUrl = gv.Visible ? "~/Img/minus.gif" : "~/Img/plus.gif";
                    imgBtn.CommandName = gv.Visible ? "collapse" : "expand";
                    break;
                case ListItemType.Header:
                    LinkButton lnk = e.Item.FindControl("lnkMName") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_NAME;

                    lnk = e.Item.FindControl("lnkMResult") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_RESULT;

                    lnk = e.Item.FindControl("lnkMU") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_MUNIT;

                    lnk = e.Item.FindControl("lnkMLL") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_LOW_LIMIT;

                    lnk = e.Item.FindControl("lnkMUL") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_HIGH_LIMIT;

                    lnk = e.Item.FindControl("lnkMPato") as LinkButton;
                    lnk.CommandArgument = DBS.F_LABTESTRESULT_PATO;

                    lnk = e.Item.FindControl("lnkMDate") as LinkButton;
                    lnk.CommandArgument = DBS.F_REQUEST_REQUEST_DATE_TIME;
                    break;
            }
        }

        protected void dtl_ItemCommand(object source, DataListCommandEventArgs e)
        {
            switch (e.CommandName.ToLower())
            { 
                case "expand":
                case "collapse":
                    GridView gv = e.Item.FindControl("gv") as GridView;
                    gv.Visible = !gv.Visible;

                    ImageButton imgBtn = e.Item.FindControl("imgBtn") as ImageButton;
                    imgBtn.Visible = gv.Rows.Count > 0;
                    imgBtn.ImageUrl = gv.Visible ? "~/Img/minus.gif" : "~/Img/plus.gif";
                    imgBtn.CommandName = gv.Visible ? "collapse" : "expand";

                    if (gv.Visible)
                    {
                        string script = "FocusControl('" + gv.ClientID + "');";
                        Page.ClientScript.RegisterStartupScript(typeof(string),"onload",script,true);
                    }
                    break;
                case "sort":
                    SortExpDtl = e.CommandArgument.ToString();
                    BindList();
                    break;
            }
        }

        protected void lnkPrint_Click(object sender, EventArgs e)
        {
            ResultsFilter filter = GetResultsFilter();
            ReportHelper.BindReport(requestResultsReport, filter);
            ExportToPDF(requestResultsReport, "Results");
        }
        #endregion

        #region Methods      

        public static DataTable GetRequestDatesWithAuthorization(UserData user)
        {
            DataTable dtRequestDates = BusinessHelper.Person.GetRequestDatesWithAuthorization(user);
            dtRequestDates.Columns.Add(new DataColumn(DBS.F_NAME, typeof(string)));
            dtRequestDates.Columns.Add(new DataColumn(DBS.F_ID, typeof(long)));

            foreach (DataRow row in dtRequestDates.Rows)
            {
                if (row.IsNull(DBS.F_REQUEST_REQUEST_DATE_TIME))
                    continue;

                row[DBS.F_NAME] = FormatMethods.GetStringDateBg(row[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                row[DBS.F_ID] = ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME]).Ticks.ToString();
            }
            dtRequestDates.DefaultView.RowFilter = DBS.F_AUTHORIZED + "=1";

            return dtRequestDates;
        }

        private void BindList()
        {
            DataView dvResults = GetMainResults();
            dtl.DataSource = dvResults;
            string sortExp = SortExpDtl;
            if (sortExp != null)
                dvResults.Sort = sortExp;

            dtl.DataBind();
        }

        private void BindGrid(GridView gv, int requestID, int serviceID)
        {
            DataView dvDetails = GetDetailedResults(requestID, serviceID);
            if (dvDetails.Count > 0)
            {
                gv.DataSource = dvDetails;
                gv.DataBind();
            }
        }

        private ResultsFilter GetResultsFilter()
        {
            ResultsFilter filter = new ResultsFilter();
            filter.EndDate = ucCalendarTo.Value;
            filter.StartDate = ucCalendarFrom.Value;
            filter.ServiceID = ParseMethods.ReadInt(cboService.SelectedValue);

            long ticks = ParseMethods.ReadLongZero(cboRequestDate.SelectedValue);
            filter.RequstDate = new DateTime(ticks);
            filter.PersonID = CurrentUserData.UserID;

            return filter;
        }

        private DataTable GetResults()
        {
            if (_dtResults == null)
            {
                ResultsFilter filter = GetResultsFilter();
                _dtResults = BusinessHelper.Person.GetResults(filter, CurrentUserData);
            } 

            return _dtResults;
        }

        private DataView GetMainResults()
        {
            DataTable dtResults = GetResults();
            DataView dv = new DataView(dtResults);
            dv.RowFilter = DBS.F_LEVEL + "=0";

            return dv;
        }

        private DataView GetDetailedResults(int requestID, int serviceID)
        {
            DataTable dtResults = GetResults();
            DataView dv = new DataView(dtResults);
            dv.RowFilter = DBS.F_LABEXAM_REQUEST_ID + "=" + requestID + " AND "
                + DBS.F_LABTEST_SERVICE_ID + "=" + serviceID + " AND "
                + DBS.F_LEVEL + ">0";

            return dv;
        }
        #endregion
    }
}
