using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using FrameWork;
using Common;
using BusinessLayer;

namespace GammaMultilabWeb
{
    public partial class Results : FrameWork.PageBase
    {
        #region Fields
        private DataTable _dtResults = null;
        private DataTable _dtRequestDates = null;
        private DataTable _dtResultRequestDates = null;
        private string SortExp
        {
            get
            {
                object sortExp = ViewState["SortExpDtl"];
                if (sortExp != null)
                    return sortExp.ToString();
                return null;
            }
            set
            {
                string sortExp = SortExp;
                sortExp = value == sortExp ? value + " DESC" : value;
                ViewState["SortExpDtl"] = sortExp;
            }
        }

        private int DesignTimeColumnCount;
        #endregion

        #region Initialization
        protected void Page_Load(object sender, EventArgs e)
        {
            
        }

        protected override void OnInit(EventArgs e)
        {
            DesignTimeColumnCount = gv.Columns.Count;
            base.OnInit(e);
        }

        protected override void LoadStaticData()
        {
            base.LoadStaticData();
            DataTable dtServices = BusinessHelper.Person.GetServices(CurrentUserData.UserID);
            UIHelper.BindCombo(cboService, dtServices, DBS.F_SERVICE_NAME, DBS.F_SERVICE_ID, true, "", Multilanguage.GetTranslatedString(PageFileName, "allservices"));

            DataTable dtRequestDates = GetRequestDates();
            UIHelper.BindCombo(cboRequestDate, dtRequestDates, DBS.F_NAME, DBS.F_ID, true, "", Multilanguage.GetTranslatedString(PageFileName, "allrequests"));
        }
        #endregion

        #region Event Handlers
        protected void btnShow_Click(object sender, EventArgs e)
        {
            gv.Attributes["binding"] = "1";
            BindGrid();
            gv.Attributes["binding"] = "0";
        }

        protected void gv_RowDataBound(object sender, DataGridItemEventArgs e)
        {
            switch (e.Item.ItemType)
            {
                case ListItemType.Item:
                case ListItemType.AlternatingItem:

                    DataRow row = (e.Item.DataItem as DataRowView).Row;
                    int activityID = ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_ACTIVITY_ID]);

                    Label lbl = e.Item.FindControl("lblName") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_NAME].ToString());
                    
                    lbl = e.Item.FindControl("lblU") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_MUNIT].ToString());
                    
                    lbl = e.Item.FindControl("lblLL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_LOW_LIMIT].ToString());

                    lbl = e.Item.FindControl("lblUL") as Label;
                    SetControlText(lbl, row[DBS.F_LABTESTRESULT_HIGH_LIMIT].ToString());

                    TableCell cell;
                    int patoState;
                    BoundColumn boundColumn;
                    for (int i = DesignTimeColumnCount; i < gv.Columns.Count; i++)
                    {
                        boundColumn = gv.Columns[i] as BoundColumn;
                        patoState = ParseMethods.ReadInt(row[DBS.F_LABTESTRESULT_PATO + '_' + boundColumn.DataField]);
                        if (patoState >= 0)
                        {
                            cell = e.Item.Cells[i];
                            cell.CssClass = patoState == (int)PatoState.None
                                ? "patoN"
                                : (patoState == (int)PatoState.Low
                                    ? "patoL"
                                    : "patoH");
                        }
                    }
                    break;
                case ListItemType.Header:
                    LinkButton lnk = e.Item.FindControl("lnkName") as LinkButton;
                    if (lnk!=null)
                        lnk.CommandArgument = DBS.F_LABTESTRESULT_NAME;

                    lnk = e.Item.FindControl("lnkU") as LinkButton;
                    if (lnk != null)
                        lnk.CommandArgument = DBS.F_LABTESTRESULT_MUNIT;

                    lnk = e.Item.FindControl("lnkLL") as LinkButton;
                    if (lnk != null)
                        lnk.CommandArgument = DBS.F_LABTESTRESULT_LOW_LIMIT;

                    lnk = e.Item.FindControl("lnkUL") as LinkButton;
                    if (lnk != null)
                        lnk.CommandArgument = DBS.F_LABTESTRESULT_HIGH_LIMIT;
                    break;
            }
        }

        protected void gv_Sorting(object sender, DataGridSortCommandEventArgs e)
        {
            SortExp = e.SortExpression;
            BindGrid();
        }

        protected void btnShowGraph_Click(object sender, EventArgs e)
        {
            Page.Validate();
            ArrayList activityIDs = GetSelectedActivities();
            if (activityIDs.Count == 0)
                ShowGeneralError(StringConstants.ERROR_NO_ACTIVITIES);
            else
            {
                string strColors = Multilanguage.GetTranslatedString(StringConstants.MESSAGES, "colors");
                if (strColors == StringConstants.NOT_TRANSLATED)
                    strColors = "#000000";
                string[] colors = strColors.Split(new char[] { ';' });
                if (activityIDs.Count > colors.Length)
                    ShowGeneralError(GetTranslatedError(StringConstants.ERROR_TOO_MANY_ACTIVITIES).Replace("%1", colors.Length.ToString()));
                else 
                { 
                    DataTable dtResults = GetResults();
                    DataRow[] drActivities;
                    string unitType = null;
                    foreach (int activityID in activityIDs)
                    {
                        
                        if ((drActivities = dtResults.Select(DBS.F_LABTESTRESULT_ACTIVITY_ID + '=' + activityID)) == null)
                            continue;
                        if (unitType != null && unitType != drActivities[0][DBS.F_LABTESTRESULT_MUNIT].ToString().ToLower().Trim())
                        {
                            ShowGeneralError(GetTranslatedError(StringConstants.ERROR_ACTIVITY_UNIT));
                            break;
                        }
                        else
                            unitType = drActivities[0][DBS.F_LABTESTRESULT_MUNIT].ToString().ToLower().Trim();
                    }
                }
            }

            if (!Page.IsValid)
                return;

            ResultsFilter resultsFilter = GetResultsFilter();

            string pageName = GetLanguageGroupKey(typeof (Reports.ActivityGraph));
            CurrentUserData.PageCache.Init(pageName);
            CurrentUserData.PageCache.Add("activityIDs", activityIDs);
            CurrentUserData.PageCache.Add("resultFilter", resultsFilter);

            string script = "ShowPopUp('" + ApplicationPath + "Reports/ActivityGraph.aspx','graph',1000,750);";
            if (!Page.ClientScript.IsStartupScriptRegistered("graph"))
                Page.ClientScript.RegisterStartupScript(typeof(string), "graph", script,true);
            BindGrid();
        }
        #endregion

        #region Methods
        private ArrayList GetSelectedActivities()
        {
            ArrayList activityIDs = new ArrayList();
            CheckBox chk;
            foreach (DataGridItem dgi in gv.Items)
            {
                chk = dgi.FindControl("chk") as CheckBox;
                if (chk.Checked)
                    activityIDs.Add(gv.DataKeys[dgi.ItemIndex]);
            }

            return activityIDs;
        }

        private void BindGrid()
        {
            ClearRuntimeColumns(gv);

            DataTable dtRequestResults = GetResults();
            DataTable dtAuthReqDates = GetResultRequestDates(dtRequestResults);
            DateTime requestDate;
            string columnName;
            BoundColumn boundColumn;
            foreach (DataRow row in dtAuthReqDates.Rows)
            {
                requestDate = ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                boundColumn = new BoundColumn();
                boundColumn.HeaderStyle.Wrap = false;
                columnName = FormatMethods.GetStringDateTimeBg(requestDate);
                boundColumn.DataField = columnName;
                boundColumn.HeaderText = columnName;
                gv.Columns.Add(boundColumn);
            }

            DataView dvResults = GetDataSource().DefaultView;
            string sortExp = SortExp;
            if (sortExp != null)
                dvResults.Sort = sortExp;
            gv.DataSource = dvResults;
            gv.DataKeyField = DBS.F_LABTESTRESULT_ACTIVITY_ID;
            gv.DataBind();
        }

        private DataTable GetDataSource()
        {
            DataTable dtRequestResult = GetResults();
            DataTable dtActivities = GetActivities(dtRequestResult);

            DataTable dtReqDates = GetResultRequestDates(dtRequestResult);
            DateTime requestDate;
            string columnName;
            int i = 0;
            foreach (DataRow row in dtReqDates.Rows)
            {
                requestDate = ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                columnName = FormatMethods.GetStringDateTimeBg(requestDate);
                if (!dtReqDates.Columns.Contains(columnName))
                {
                    dtActivities.Columns.Add(new DataColumn(columnName, typeof(string)));
                    dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_PATO + '_' + columnName, typeof(string)));
                    i++;
                }
            }

            int activityID;
            DataRow[] rows;
            foreach (DataRow activityRow in dtActivities.Rows)
            {
                activityID = ParseMethods.ReadInt(activityRow[DBS.F_LABTESTRESULT_ACTIVITY_ID]);
                rows = dtRequestResult.Select(DBS.F_LABTESTRESULT_ACTIVITY_ID + "=" + activityID);
                foreach (DataRow row in rows)
                {
                    requestDate = ParseMethods.ReadDate(row[DBS.F_REQUEST_REQUEST_DATE_TIME]);
                    columnName = FormatMethods.GetStringDateTimeBg(requestDate);
                    activityRow[columnName] = row[DBS.F_LABTESTRESULT_RESULT];
                    activityRow[DBS.F_LABTESTRESULT_PATO + '_' + columnName] = row[DBS.F_LABTESTRESULT_PATO];
                }
            }
            return dtActivities;
        }

        private DataTable GetActivities(DataTable dtRequestResult)
        {
            DataTable dtActivities = new DataTable("Activity");
            dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_ACTIVITY_ID, typeof(int)));
            dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_NAME, typeof(string)));
            dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_MUNIT, typeof(string)));
            dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_LOW_LIMIT, typeof(string)));
            dtActivities.Columns.Add(new DataColumn(DBS.F_LABTESTRESULT_HIGH_LIMIT, typeof(string)));

            dtActivities.PrimaryKey = new DataColumn[] { dtActivities.Columns[DBS.F_LABTESTRESULT_ACTIVITY_ID] };

            DataRow row;
            foreach (DataRowView drv in dtRequestResult.DefaultView)
            {
                row = drv.Row;
                if (!row.IsNull(DBS.F_LABTESTRESULT_ACTIVITY_ID) && dtActivities.Rows.Find(row[DBS.F_LABTESTRESULT_ACTIVITY_ID]) == null)
                    dtActivities.Rows.Add(new object[]{
                        row[DBS.F_LABTESTRESULT_ACTIVITY_ID],
                        row[DBS.F_LABTESTRESULT_NAME],
                        row[DBS.F_LABTESTRESULT_MUNIT],
                        row[DBS.F_LABTESTRESULT_LOW_LIMIT],
                        row[DBS.F_LABTESTRESULT_HIGH_LIMIT]
                    });
            }

            return dtActivities;
        }

        private void ClearRuntimeColumns(DataGrid gv)
        {
            while (gv.Columns.Count > DesignTimeColumnCount)
                gv.Columns.RemoveAt(gv.Columns.Count-1);
        }

        private ResultsFilter GetResultsFilter()
        {
            ResultsFilter filter = new ResultsFilter();
            filter.EndDate = ucCalendarTo.Value;
            filter.StartDate = ucCalendarFrom.Value;
            filter.ServiceID = ParseMethods.ReadInt(cboService.SelectedValue);

            long ticks = ParseMethods.ReadLongZero(cboRequestDate.SelectedValue);
            filter.RequstDate = new DateTime(ticks);
            filter.PersonID = CurrentUserData.UserID;

            return filter;
        }

        private DataTable GetResults()
        {
            if (_dtResults == null)
            {
                DataSet ds = CurrentUserData.PageCache["Results"] as DataSet;
                if (ds!=null && ds.Tables.Count > 0 && !ParseMethods.ReadBool(gv.Attributes["binding"]))
                    _dtResults = ds.Tables[0];
                else
                {
                    ResultsFilter filter = GetResultsFilter();
                    _dtResults = BusinessHelper.Person.GetResults(filter, CurrentUserData);
                    if (ds == null)
                        ds = new DataSet();
                    else
                        ds.Tables.Clear();
                    if (ds.Tables.Count == 0)
                    {
                        ds.Tables.Add(_dtResults);
                        CurrentUserData.PageCache.Add("Results", ds);
                    }
                }
            }

            return _dtResults;
        }

        private DataTable GetRequestDates()
        {
            if (_dtRequestDates == null)
                _dtRequestDates = Requests.GetRequestDatesWithAuthorization(CurrentUserData);
            return _dtRequestDates;
        }

        //Result request date time can differ from request dates
        //By default the result is ordered by RequestDateTime
        private DataTable GetResultRequestDates(DataTable dtRequestResults)
        {
            if (_dtResultRequestDates == null)
                _dtResultRequestDates = BusinessHelper.Person.GetResultRequestDates(dtRequestResults); ;
            return _dtResultRequestDates;
        }
        #endregion
    }
}
